#pragma once

#include "NPCommonUI.h"
#include <stack>

class AbstractModel;
class AbstractView;
class AbstractCommand;

class NPCOMMONUI_API AbstractControl
{
public:
	AbstractControl(void);
	virtual ~AbstractControl(void);

	/**
	* @brief	initialize
	*/
	virtual void Init() = 0;
	/**
	* @brief	uninitialize
	*/
	virtual void Close() = 0;
	/**
	* @brief	set view pointer
	* @param	pView	pointer of view
	* @return	none
	*/
	void	SetView(AbstractView* view);
	/**
	* @brief	set model pointer
	* @param	pModel	pointer of model
	* @return	none
	*/
	void	SetModel(AbstractModel* model);
	/**
	* @brief	retrieve view base
	* @return	if function succeeds, the return value is AbstractView pointer
	*/
	AbstractView*	GetView() { return m_View; }
	/**
	* @brief	retrieve model base
	* @return	if function succeeds, the return value is AbstractModel pointer
	*/
	AbstractModel*	GetModel() { return m_Model; }
	
	/**
	* @brief	push undo command
	* @param	pCommand undo command
	* @param	BOOL bRemoveRedo
	* @return	BOOL
	*/
	BOOL PushUndoCommand(AbstractCommand* command, BOOL bRemoveRedo=TRUE);
	/**
	* @brief	push redo command
	* @param	pCommand redo command
	* @return	BOOL
	*/
	BOOL PushRedoCommand(AbstractCommand* command);
	/**
	* @brief	pop undo command
	* @return	AbstractCommand*
	*/
	AbstractCommand* PopUndoCommand();
	AbstractCommand* PeekUndoCommand();
	/**
	* @brief	pop redo command
	* @return	AbstractCommand*
	*/
	AbstractCommand* PopRedoCommand();
	AbstractCommand* PeekRedoCommand();
	/**
	* @brief	reset
	* @return	none
	*/
	void Reset();

	/**
	* @brief	count of undo stack 
	* @return	INT32
	*/
	INT32 GetUndoCount() { return m_stackUndo.size(); }
	/**
	* @brief	count of redo stack 
	* @return	INT32
	*/
	INT32 GetRedoCount() { return m_stackRedo.size(); }
private:
	///< view
	AbstractView*	m_View;
	///< model
	AbstractModel*	m_Model;

	// stack undo command
	std::stack<AbstractCommand*>	m_stackUndo;
	// stack redo command
	std::stack<AbstractCommand*>	m_stackRedo;

	void StackRemoveAll(std::stack<AbstractCommand*>& stackCommand);
};
